#include <pspkernel.h>
#include <psppower.h>
#include <systemctrl.h> 



PSP_MODULE_INFO("CtrlCharge", PSP_MODULE_KERNEL, 1, 0);



typedef struct FuncList
{
	int (*scePowerIsBatteryExist)(void);
	int (*scePowerIsBatteryCharging)(void);
	int (*scePowerGetBatteryLifePercent)(void);
	void (*sceSysconPermitChargeBattery)(void);
	void (*sceSysconForbidChargeBattery)(void);
}FuncList;



static int stopFlag;



static void GetFuncs(FuncList *funcs)
{
	const u32 nids [][5] = {
		{ 0x0AFD0D8B, 0x1E490401, 0x2085D15D, 0xEAB13FBE, 0xC5075828 }, // hash
//		{ 0x1A4EC14D, 0x0AFD8E3A, 0xE2552927, 0x4EB03528, 0xBF742292 }, // 5.00
		{ 0xBC8823E8, 0x5202A826, 0xE5F8596E, 0x4EB03528, 0x027785AD }, // 6.20
		{ 0x81DCA5A5, 0xAB842949, 0x2CBFA597, 0x97745810, 0x95B64473 }  // 6.35
	};

	int (*KernelDevkitVersion)(void);
	int version, index;

	KernelDevkitVersion = (int(*)(void))FindProc("sceSystemMemoryManager", "SysMemUserForUser", 0x3FC9AE6A);
	version = KernelDevkitVersion();

	switch(version)
	{
	case 0x06020510: // 6.20
		index = 2 - 1;
		break;

	case 0x06030510: // 6.35
		index = 3 - 1;
		break;

	default:
		index = 0;
		break;
	}

	funcs->scePowerIsBatteryExist = (int(*)(void))FindProc("scePower_Service", "scePower_driver", nids[index][0]);
	funcs->scePowerIsBatteryCharging = (int(*)(void))FindProc("scePower_Service", "scePower_driver", nids[index][1]);
	funcs->scePowerGetBatteryLifePercent = (int(*)(void))FindProc("scePower_Service", "scePower_driver", nids[index][2]);
	funcs->sceSysconPermitChargeBattery = (void(*)(void))FindProc("sceSYSCON_Driver", "sceSyscon_driver", nids[index][3]);
	funcs->sceSysconForbidChargeBattery = (void(*)(void))FindProc("sceSYSCON_Driver", "sceSyscon_driver", nids[index][4]);
}

static int main_thread(SceSize args, void *argp)
{
	FuncList funcs;
	int isCharging;

	// wait module
	while(1)
	{
		if(sceKernelFindModuleByName("scePower_Service") != NULL
			&& sceKernelFindModuleByName("sceSYSCON_Driver") != NULL)
		{
			break;
		}

		sceKernelDelayThread(10000);
	}

	// resolve nid
	GetFuncs(&funcs);

	while(stopFlag == 0)
	{
		if(funcs.scePowerIsBatteryExist() == 1)
		{
			isCharging = funcs.scePowerIsBatteryCharging();

			if(isCharging == 0)	// isn't charging
			{
				if(funcs.scePowerGetBatteryLifePercent() < 70)
					funcs.sceSysconPermitChargeBattery(); // start charging
			}
			else if(isCharging == 1) // is charging
			{
				if(funcs.scePowerGetBatteryLifePercent() >= 70)
					funcs.sceSysconForbidChargeBattery(); // stop charging
			}
		}

		// wait 5 seconds
		sceKernelDelayThread(5 * 1000 * 1000);
	}

	sceKernelExitDeleteThread(0);
	return 0;
}

int module_start(SceSize args, void *argp)
{
	SceUID thid;

	thid = sceKernelCreateThread("ctrl_charge", main_thread, 0x18, 0x500, 0, NULL);
	if(thid >= 0)
		sceKernelStartThread(thid, 0, NULL);

	return 0;
}

int module_stop(SceSize args, void *argp)
{
	stopFlag = 1;
	return 0;
}